# coding: utf-8

"""
    Kill Bill

    Kill Bill is an open-source billing and payments platform  # noqa: E501

    OpenAPI spec version: 0.22.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Subject(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'principal': 'str',
        'is_authenticated': 'bool',
        'is_remembered': 'bool',
        'session': 'Session'
    }

    attribute_map = {
        'principal': 'principal',
        'is_authenticated': 'isAuthenticated',
        'is_remembered': 'isRemembered',
        'session': 'session'
    }

    def __init__(self, principal=None, is_authenticated=None, is_remembered=None, session=None):  # noqa: E501
        """Subject - a model defined in Swagger"""  # noqa: E501
        self._principal = None
        self._is_authenticated = None
        self._is_remembered = None
        self._session = None
        self.discriminator = None
        if principal is not None:
            self.principal = principal
        if is_authenticated is not None:
            self.is_authenticated = is_authenticated
        if is_remembered is not None:
            self.is_remembered = is_remembered
        if session is not None:
            self.session = session

    @property
    def principal(self):
        """Gets the principal of this Subject.  # noqa: E501


        :return: The principal of this Subject.  # noqa: E501
        :rtype: str
        """
        return self._principal

    @principal.setter
    def principal(self, principal):
        """Sets the principal of this Subject.


        :param principal: The principal of this Subject.  # noqa: E501
        :type: str
        """

        self._principal = principal

    @property
    def is_authenticated(self):
        """Gets the is_authenticated of this Subject.  # noqa: E501


        :return: The is_authenticated of this Subject.  # noqa: E501
        :rtype: bool
        """
        return self._is_authenticated

    @is_authenticated.setter
    def is_authenticated(self, is_authenticated):
        """Sets the is_authenticated of this Subject.


        :param is_authenticated: The is_authenticated of this Subject.  # noqa: E501
        :type: bool
        """

        self._is_authenticated = is_authenticated

    @property
    def is_remembered(self):
        """Gets the is_remembered of this Subject.  # noqa: E501


        :return: The is_remembered of this Subject.  # noqa: E501
        :rtype: bool
        """
        return self._is_remembered

    @is_remembered.setter
    def is_remembered(self, is_remembered):
        """Sets the is_remembered of this Subject.


        :param is_remembered: The is_remembered of this Subject.  # noqa: E501
        :type: bool
        """

        self._is_remembered = is_remembered

    @property
    def session(self):
        """Gets the session of this Subject.  # noqa: E501


        :return: The session of this Subject.  # noqa: E501
        :rtype: Session
        """
        return self._session

    @session.setter
    def session(self, session):
        """Sets the session of this Subject.


        :param session: The session of this Subject.  # noqa: E501
        :type: Session
        """

        self._session = session

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Subject, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Subject):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Kill Bill

    Kill Bill is an open-source billing and payments platform  # noqa: E501

    OpenAPI spec version: 0.22.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from killbill.api_client import ApiClient


class SubscriptionApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def add_subscription_blocking_state(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """Block a subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_subscription_blocking_state(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param BlockingState body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date requested_date:
        :param list[str] plugin_property:
        :return: list[BlockingState]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_subscription_blocking_state_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
        else:
            (data) = self.add_subscription_blocking_state_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
            return data

    def add_subscription_blocking_state_with_http_info(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """Block a subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_subscription_blocking_state_with_http_info(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param BlockingState body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date requested_date:
        :param list[str] plugin_property:
        :return: list[BlockingState]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'x_killbill_created_by', 'subscription_id', 'x_killbill_reason', 'x_killbill_comment', 'requested_date', 'plugin_property']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_subscription_blocking_state" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `add_subscription_blocking_state`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `add_subscription_blocking_state`")  # noqa: E501
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `add_subscription_blocking_state`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []
        if 'requested_date' in params:
            query_params.append(('requestedDate', params['requested_date']))  # noqa: E501
        if 'plugin_property' in params:
            query_params.append(('pluginProperty', params['plugin_property']))  # noqa: E501
            collection_formats['pluginProperty'] = 'multi'  # noqa: E501

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/block', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[BlockingState]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cancel_subscription_plan(self, subscription_id, x_killbill_created_by, **kwargs):  # noqa: E501
        """Cancel an entitlement plan  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cancel_subscription_plan(subscription_id, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str x_killbill_created_by: (required)
        :param date requested_date:
        :param bool call_completion:
        :param int call_timeout_sec:
        :param str entitlement_policy:
        :param str billing_policy:
        :param bool use_requested_date_for_billing:
        :param list[str] plugin_property:
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cancel_subscription_plan_with_http_info(subscription_id, x_killbill_created_by, **kwargs)  # noqa: E501
        else:
            (data) = self.cancel_subscription_plan_with_http_info(subscription_id, x_killbill_created_by, **kwargs)  # noqa: E501
            return data

    def cancel_subscription_plan_with_http_info(self, subscription_id, x_killbill_created_by, **kwargs):  # noqa: E501
        """Cancel an entitlement plan  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cancel_subscription_plan_with_http_info(subscription_id, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str x_killbill_created_by: (required)
        :param date requested_date:
        :param bool call_completion:
        :param int call_timeout_sec:
        :param str entitlement_policy:
        :param str billing_policy:
        :param bool use_requested_date_for_billing:
        :param list[str] plugin_property:
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['subscription_id', 'x_killbill_created_by', 'requested_date', 'call_completion', 'call_timeout_sec', 'entitlement_policy', 'billing_policy', 'use_requested_date_for_billing', 'plugin_property', 'x_killbill_reason', 'x_killbill_comment']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cancel_subscription_plan" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `cancel_subscription_plan`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `cancel_subscription_plan`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []
        if 'requested_date' in params:
            query_params.append(('requestedDate', params['requested_date']))  # noqa: E501
        if 'call_completion' in params:
            query_params.append(('callCompletion', params['call_completion']))  # noqa: E501
        if 'call_timeout_sec' in params:
            query_params.append(('callTimeoutSec', params['call_timeout_sec']))  # noqa: E501
        if 'entitlement_policy' in params:
            query_params.append(('entitlementPolicy', params['entitlement_policy']))  # noqa: E501
        if 'billing_policy' in params:
            query_params.append(('billingPolicy', params['billing_policy']))  # noqa: E501
        if 'use_requested_date_for_billing' in params:
            query_params.append(('useRequestedDateForBilling', params['use_requested_date_for_billing']))  # noqa: E501
        if 'plugin_property' in params:
            query_params.append(('pluginProperty', params['plugin_property']))  # noqa: E501
            collection_formats['pluginProperty'] = 'multi'  # noqa: E501

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def change_subscription_plan(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """Change entitlement plan  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.change_subscription_plan(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Subscription body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date requested_date:
        :param bool call_completion:
        :param int call_timeout_sec:
        :param str billing_policy:
        :param list[str] plugin_property:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.change_subscription_plan_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
        else:
            (data) = self.change_subscription_plan_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
            return data

    def change_subscription_plan_with_http_info(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """Change entitlement plan  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.change_subscription_plan_with_http_info(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Subscription body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date requested_date:
        :param bool call_completion:
        :param int call_timeout_sec:
        :param str billing_policy:
        :param list[str] plugin_property:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'x_killbill_created_by', 'subscription_id', 'x_killbill_reason', 'x_killbill_comment', 'requested_date', 'call_completion', 'call_timeout_sec', 'billing_policy', 'plugin_property']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method change_subscription_plan" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `change_subscription_plan`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `change_subscription_plan`")  # noqa: E501
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `change_subscription_plan`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []
        if 'requested_date' in params:
            query_params.append(('requestedDate', params['requested_date']))  # noqa: E501
        if 'call_completion' in params:
            query_params.append(('callCompletion', params['call_completion']))  # noqa: E501
        if 'call_timeout_sec' in params:
            query_params.append(('callTimeoutSec', params['call_timeout_sec']))  # noqa: E501
        if 'billing_policy' in params:
            query_params.append(('billingPolicy', params['billing_policy']))  # noqa: E501
        if 'plugin_property' in params:
            query_params.append(('pluginProperty', params['plugin_property']))  # noqa: E501
            collection_formats['pluginProperty'] = 'multi'  # noqa: E501

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_subscription(self, body, x_killbill_created_by, **kwargs):  # noqa: E501
        """Create an subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_subscription(body, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Subscription body: (required)
        :param str x_killbill_created_by: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date entitlement_date:
        :param date billing_date:
        :param bool rename_key_if_exists_and_unused:
        :param bool migrated:
        :param bool call_completion:
        :param int call_timeout_sec:
        :param list[str] plugin_property:
        :return: Subscription
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_subscription_with_http_info(body, x_killbill_created_by, **kwargs)  # noqa: E501
        else:
            (data) = self.create_subscription_with_http_info(body, x_killbill_created_by, **kwargs)  # noqa: E501
            return data

    def create_subscription_with_http_info(self, body, x_killbill_created_by, **kwargs):  # noqa: E501
        """Create an subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_subscription_with_http_info(body, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Subscription body: (required)
        :param str x_killbill_created_by: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date entitlement_date:
        :param date billing_date:
        :param bool rename_key_if_exists_and_unused:
        :param bool migrated:
        :param bool call_completion:
        :param int call_timeout_sec:
        :param list[str] plugin_property:
        :return: Subscription
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'x_killbill_created_by', 'x_killbill_reason', 'x_killbill_comment', 'entitlement_date', 'billing_date', 'rename_key_if_exists_and_unused', 'migrated', 'call_completion', 'call_timeout_sec', 'plugin_property']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_subscription" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_subscription`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `create_subscription`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'entitlement_date' in params:
            query_params.append(('entitlementDate', params['entitlement_date']))  # noqa: E501
        if 'billing_date' in params:
            query_params.append(('billingDate', params['billing_date']))  # noqa: E501
        if 'rename_key_if_exists_and_unused' in params:
            query_params.append(('renameKeyIfExistsAndUnused', params['rename_key_if_exists_and_unused']))  # noqa: E501
        if 'migrated' in params:
            query_params.append(('migrated', params['migrated']))  # noqa: E501
        if 'call_completion' in params:
            query_params.append(('callCompletion', params['call_completion']))  # noqa: E501
        if 'call_timeout_sec' in params:
            query_params.append(('callTimeoutSec', params['call_timeout_sec']))  # noqa: E501
        if 'plugin_property' in params:
            query_params.append(('pluginProperty', params['plugin_property']))  # noqa: E501
            collection_formats['pluginProperty'] = 'multi'  # noqa: E501

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Subscription',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_subscription_custom_fields(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """Add custom fields to subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_subscription_custom_fields(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[CustomField] body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_subscription_custom_fields_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
        else:
            (data) = self.create_subscription_custom_fields_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
            return data

    def create_subscription_custom_fields_with_http_info(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """Add custom fields to subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_subscription_custom_fields_with_http_info(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[CustomField] body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'x_killbill_created_by', 'subscription_id', 'x_killbill_reason', 'x_killbill_comment']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_subscription_custom_fields" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_subscription_custom_fields`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `create_subscription_custom_fields`")  # noqa: E501
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `create_subscription_custom_fields`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/customFields', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_subscription_tags(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """create_subscription_tags  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_subscription_tags(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[str] body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_subscription_tags_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
        else:
            (data) = self.create_subscription_tags_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
            return data

    def create_subscription_tags_with_http_info(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """create_subscription_tags  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_subscription_tags_with_http_info(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[str] body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'x_killbill_created_by', 'subscription_id', 'x_killbill_reason', 'x_killbill_comment']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_subscription_tags" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_subscription_tags`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `create_subscription_tags`")  # noqa: E501
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `create_subscription_tags`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/tags', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_subscription_with_add_ons(self, body, x_killbill_created_by, **kwargs):  # noqa: E501
        """Create an entitlement with addOn products  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_subscription_with_add_ons(body, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[Subscription] body: (required)
        :param str x_killbill_created_by: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date entitlement_date:
        :param date billing_date:
        :param bool migrated:
        :param bool rename_key_if_exists_and_unused:
        :param bool call_completion:
        :param int call_timeout_sec:
        :param list[str] plugin_property:
        :return: Bundle
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_subscription_with_add_ons_with_http_info(body, x_killbill_created_by, **kwargs)  # noqa: E501
        else:
            (data) = self.create_subscription_with_add_ons_with_http_info(body, x_killbill_created_by, **kwargs)  # noqa: E501
            return data

    def create_subscription_with_add_ons_with_http_info(self, body, x_killbill_created_by, **kwargs):  # noqa: E501
        """Create an entitlement with addOn products  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_subscription_with_add_ons_with_http_info(body, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[Subscription] body: (required)
        :param str x_killbill_created_by: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date entitlement_date:
        :param date billing_date:
        :param bool migrated:
        :param bool rename_key_if_exists_and_unused:
        :param bool call_completion:
        :param int call_timeout_sec:
        :param list[str] plugin_property:
        :return: Bundle
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'x_killbill_created_by', 'x_killbill_reason', 'x_killbill_comment', 'entitlement_date', 'billing_date', 'migrated', 'rename_key_if_exists_and_unused', 'call_completion', 'call_timeout_sec', 'plugin_property']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_subscription_with_add_ons" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_subscription_with_add_ons`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `create_subscription_with_add_ons`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'entitlement_date' in params:
            query_params.append(('entitlementDate', params['entitlement_date']))  # noqa: E501
        if 'billing_date' in params:
            query_params.append(('billingDate', params['billing_date']))  # noqa: E501
        if 'migrated' in params:
            query_params.append(('migrated', params['migrated']))  # noqa: E501
        if 'rename_key_if_exists_and_unused' in params:
            query_params.append(('renameKeyIfExistsAndUnused', params['rename_key_if_exists_and_unused']))  # noqa: E501
        if 'call_completion' in params:
            query_params.append(('callCompletion', params['call_completion']))  # noqa: E501
        if 'call_timeout_sec' in params:
            query_params.append(('callTimeoutSec', params['call_timeout_sec']))  # noqa: E501
        if 'plugin_property' in params:
            query_params.append(('pluginProperty', params['plugin_property']))  # noqa: E501
            collection_formats['pluginProperty'] = 'multi'  # noqa: E501

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/createSubscriptionWithAddOns', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Bundle',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_subscriptions_with_add_ons(self, body, x_killbill_created_by, **kwargs):  # noqa: E501
        """Create multiple entitlements with addOn products  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_subscriptions_with_add_ons(body, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[BulkSubscriptionsBundle] body: (required)
        :param str x_killbill_created_by: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date entitlement_date:
        :param date billing_date:
        :param bool rename_key_if_exists_and_unused:
        :param bool migrated:
        :param bool call_completion:
        :param int call_timeout_sec:
        :param list[str] plugin_property:
        :return: list[Bundle]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_subscriptions_with_add_ons_with_http_info(body, x_killbill_created_by, **kwargs)  # noqa: E501
        else:
            (data) = self.create_subscriptions_with_add_ons_with_http_info(body, x_killbill_created_by, **kwargs)  # noqa: E501
            return data

    def create_subscriptions_with_add_ons_with_http_info(self, body, x_killbill_created_by, **kwargs):  # noqa: E501
        """Create multiple entitlements with addOn products  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_subscriptions_with_add_ons_with_http_info(body, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[BulkSubscriptionsBundle] body: (required)
        :param str x_killbill_created_by: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date entitlement_date:
        :param date billing_date:
        :param bool rename_key_if_exists_and_unused:
        :param bool migrated:
        :param bool call_completion:
        :param int call_timeout_sec:
        :param list[str] plugin_property:
        :return: list[Bundle]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'x_killbill_created_by', 'x_killbill_reason', 'x_killbill_comment', 'entitlement_date', 'billing_date', 'rename_key_if_exists_and_unused', 'migrated', 'call_completion', 'call_timeout_sec', 'plugin_property']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_subscriptions_with_add_ons" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_subscriptions_with_add_ons`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `create_subscriptions_with_add_ons`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'entitlement_date' in params:
            query_params.append(('entitlementDate', params['entitlement_date']))  # noqa: E501
        if 'billing_date' in params:
            query_params.append(('billingDate', params['billing_date']))  # noqa: E501
        if 'rename_key_if_exists_and_unused' in params:
            query_params.append(('renameKeyIfExistsAndUnused', params['rename_key_if_exists_and_unused']))  # noqa: E501
        if 'migrated' in params:
            query_params.append(('migrated', params['migrated']))  # noqa: E501
        if 'call_completion' in params:
            query_params.append(('callCompletion', params['call_completion']))  # noqa: E501
        if 'call_timeout_sec' in params:
            query_params.append(('callTimeoutSec', params['call_timeout_sec']))  # noqa: E501
        if 'plugin_property' in params:
            query_params.append(('pluginProperty', params['plugin_property']))  # noqa: E501
            collection_formats['pluginProperty'] = 'multi'  # noqa: E501

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/createSubscriptionsWithAddOns', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[Bundle]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_subscription_custom_fields(self, subscription_id, x_killbill_created_by, **kwargs):  # noqa: E501
        """Remove custom fields from subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_subscription_custom_fields(subscription_id, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str x_killbill_created_by: (required)
        :param list[str] custom_field:
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_subscription_custom_fields_with_http_info(subscription_id, x_killbill_created_by, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_subscription_custom_fields_with_http_info(subscription_id, x_killbill_created_by, **kwargs)  # noqa: E501
            return data

    def delete_subscription_custom_fields_with_http_info(self, subscription_id, x_killbill_created_by, **kwargs):  # noqa: E501
        """Remove custom fields from subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_subscription_custom_fields_with_http_info(subscription_id, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str x_killbill_created_by: (required)
        :param list[str] custom_field:
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['subscription_id', 'x_killbill_created_by', 'custom_field', 'x_killbill_reason', 'x_killbill_comment']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_subscription_custom_fields" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `delete_subscription_custom_fields`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `delete_subscription_custom_fields`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []
        if 'custom_field' in params:
            query_params.append(('customField', params['custom_field']))  # noqa: E501
            collection_formats['customField'] = 'multi'  # noqa: E501

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/customFields', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_subscription_tags(self, subscription_id, x_killbill_created_by, **kwargs):  # noqa: E501
        """Remove tags from subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_subscription_tags(subscription_id, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str x_killbill_created_by: (required)
        :param list[str] tag_def:
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_subscription_tags_with_http_info(subscription_id, x_killbill_created_by, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_subscription_tags_with_http_info(subscription_id, x_killbill_created_by, **kwargs)  # noqa: E501
            return data

    def delete_subscription_tags_with_http_info(self, subscription_id, x_killbill_created_by, **kwargs):  # noqa: E501
        """Remove tags from subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_subscription_tags_with_http_info(subscription_id, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str x_killbill_created_by: (required)
        :param list[str] tag_def:
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['subscription_id', 'x_killbill_created_by', 'tag_def', 'x_killbill_reason', 'x_killbill_comment']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_subscription_tags" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `delete_subscription_tags`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `delete_subscription_tags`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []
        if 'tag_def' in params:
            query_params.append(('tagDef', params['tag_def']))  # noqa: E501
            collection_formats['tagDef'] = 'multi'  # noqa: E501

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/tags', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_subscription(self, subscription_id, **kwargs):  # noqa: E501
        """Retrieve a subscription by id  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription(subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str audit:
        :return: Subscription
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_subscription_with_http_info(subscription_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_subscription_with_http_info(subscription_id, **kwargs)  # noqa: E501
            return data

    def get_subscription_with_http_info(self, subscription_id, **kwargs):  # noqa: E501
        """Retrieve a subscription by id  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription_with_http_info(subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str audit:
        :return: Subscription
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['subscription_id', 'audit']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_subscription" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `get_subscription`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []
        if 'audit' in params:
            query_params.append(('audit', params['audit']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Subscription',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_subscription_audit_logs_with_history(self, subscription_id, **kwargs):  # noqa: E501
        """Retrieve subscription audit logs with history by id  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription_audit_logs_with_history(subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :return: list[AuditLog]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_subscription_audit_logs_with_history_with_http_info(subscription_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_subscription_audit_logs_with_history_with_http_info(subscription_id, **kwargs)  # noqa: E501
            return data

    def get_subscription_audit_logs_with_history_with_http_info(self, subscription_id, **kwargs):  # noqa: E501
        """Retrieve subscription audit logs with history by id  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription_audit_logs_with_history_with_http_info(subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :return: list[AuditLog]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['subscription_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_subscription_audit_logs_with_history" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `get_subscription_audit_logs_with_history`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/auditLogsWithHistory', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[AuditLog]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_subscription_by_key(self, external_key, **kwargs):  # noqa: E501
        """Retrieve a subscription by external key  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription_by_key(external_key, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str external_key: (required)
        :param str audit:
        :return: Subscription
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_subscription_by_key_with_http_info(external_key, **kwargs)  # noqa: E501
        else:
            (data) = self.get_subscription_by_key_with_http_info(external_key, **kwargs)  # noqa: E501
            return data

    def get_subscription_by_key_with_http_info(self, external_key, **kwargs):  # noqa: E501
        """Retrieve a subscription by external key  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription_by_key_with_http_info(external_key, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str external_key: (required)
        :param str audit:
        :return: Subscription
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['external_key', 'audit']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_subscription_by_key" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'external_key' is set
        if ('external_key' not in params or
                params['external_key'] is None):
            raise ValueError("Missing the required parameter `external_key` when calling `get_subscription_by_key`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'external_key' in params:
            query_params.append(('externalKey', params['external_key']))  # noqa: E501
        if 'audit' in params:
            query_params.append(('audit', params['audit']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Subscription',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_subscription_custom_fields(self, subscription_id, **kwargs):  # noqa: E501
        """Retrieve subscription custom fields  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription_custom_fields(subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str audit:
        :return: list[CustomField]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_subscription_custom_fields_with_http_info(subscription_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_subscription_custom_fields_with_http_info(subscription_id, **kwargs)  # noqa: E501
            return data

    def get_subscription_custom_fields_with_http_info(self, subscription_id, **kwargs):  # noqa: E501
        """Retrieve subscription custom fields  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription_custom_fields_with_http_info(subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str audit:
        :return: list[CustomField]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['subscription_id', 'audit']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_subscription_custom_fields" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `get_subscription_custom_fields`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []
        if 'audit' in params:
            query_params.append(('audit', params['audit']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/customFields', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[CustomField]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_subscription_event_audit_logs_with_history(self, event_id, **kwargs):  # noqa: E501
        """Retrieve subscription event audit logs with history by id  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription_event_audit_logs_with_history(event_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str event_id: (required)
        :return: list[AuditLog]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_subscription_event_audit_logs_with_history_with_http_info(event_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_subscription_event_audit_logs_with_history_with_http_info(event_id, **kwargs)  # noqa: E501
            return data

    def get_subscription_event_audit_logs_with_history_with_http_info(self, event_id, **kwargs):  # noqa: E501
        """Retrieve subscription event audit logs with history by id  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription_event_audit_logs_with_history_with_http_info(event_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str event_id: (required)
        :return: list[AuditLog]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['event_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_subscription_event_audit_logs_with_history" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'event_id' is set
        if ('event_id' not in params or
                params['event_id'] is None):
            raise ValueError("Missing the required parameter `event_id` when calling `get_subscription_event_audit_logs_with_history`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'event_id' in params:
            path_params['eventId'] = params['event_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/events/{eventId}/auditLogsWithHistory', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[AuditLog]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_subscription_tags(self, subscription_id, **kwargs):  # noqa: E501
        """Retrieve subscription tags  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription_tags(subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param bool included_deleted:
        :param str audit:
        :return: list[Tag]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_subscription_tags_with_http_info(subscription_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_subscription_tags_with_http_info(subscription_id, **kwargs)  # noqa: E501
            return data

    def get_subscription_tags_with_http_info(self, subscription_id, **kwargs):  # noqa: E501
        """Retrieve subscription tags  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_subscription_tags_with_http_info(subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param bool included_deleted:
        :param str audit:
        :return: list[Tag]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['subscription_id', 'included_deleted', 'audit']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_subscription_tags" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `get_subscription_tags`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []
        if 'included_deleted' in params:
            query_params.append(('includedDeleted', params['included_deleted']))  # noqa: E501
        if 'audit' in params:
            query_params.append(('audit', params['audit']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/tags', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[Tag]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def modify_subscription_custom_fields(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """Modify custom fields to subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.modify_subscription_custom_fields(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[CustomField] body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.modify_subscription_custom_fields_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
        else:
            (data) = self.modify_subscription_custom_fields_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
            return data

    def modify_subscription_custom_fields_with_http_info(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """Modify custom fields to subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.modify_subscription_custom_fields_with_http_info(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[CustomField] body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'x_killbill_created_by', 'subscription_id', 'x_killbill_reason', 'x_killbill_comment']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method modify_subscription_custom_fields" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `modify_subscription_custom_fields`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `modify_subscription_custom_fields`")  # noqa: E501
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `modify_subscription_custom_fields`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/customFields', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def uncancel_subscription_plan(self, subscription_id, x_killbill_created_by, **kwargs):  # noqa: E501
        """Un-cancel an entitlement  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.uncancel_subscription_plan(subscription_id, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str x_killbill_created_by: (required)
        :param list[str] plugin_property:
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.uncancel_subscription_plan_with_http_info(subscription_id, x_killbill_created_by, **kwargs)  # noqa: E501
        else:
            (data) = self.uncancel_subscription_plan_with_http_info(subscription_id, x_killbill_created_by, **kwargs)  # noqa: E501
            return data

    def uncancel_subscription_plan_with_http_info(self, subscription_id, x_killbill_created_by, **kwargs):  # noqa: E501
        """Un-cancel an entitlement  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.uncancel_subscription_plan_with_http_info(subscription_id, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str x_killbill_created_by: (required)
        :param list[str] plugin_property:
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['subscription_id', 'x_killbill_created_by', 'plugin_property', 'x_killbill_reason', 'x_killbill_comment']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method uncancel_subscription_plan" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `uncancel_subscription_plan`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `uncancel_subscription_plan`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []
        if 'plugin_property' in params:
            query_params.append(('pluginProperty', params['plugin_property']))  # noqa: E501
            collection_formats['pluginProperty'] = 'multi'  # noqa: E501

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/uncancel', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def undo_change_subscription_plan(self, subscription_id, x_killbill_created_by, **kwargs):  # noqa: E501
        """Undo a pending change plan on an entitlement  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.undo_change_subscription_plan(subscription_id, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str x_killbill_created_by: (required)
        :param list[str] plugin_property:
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.undo_change_subscription_plan_with_http_info(subscription_id, x_killbill_created_by, **kwargs)  # noqa: E501
        else:
            (data) = self.undo_change_subscription_plan_with_http_info(subscription_id, x_killbill_created_by, **kwargs)  # noqa: E501
            return data

    def undo_change_subscription_plan_with_http_info(self, subscription_id, x_killbill_created_by, **kwargs):  # noqa: E501
        """Undo a pending change plan on an entitlement  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.undo_change_subscription_plan_with_http_info(subscription_id, x_killbill_created_by, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str subscription_id: (required)
        :param str x_killbill_created_by: (required)
        :param list[str] plugin_property:
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['subscription_id', 'x_killbill_created_by', 'plugin_property', 'x_killbill_reason', 'x_killbill_comment']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method undo_change_subscription_plan" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `undo_change_subscription_plan`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `undo_change_subscription_plan`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []
        if 'plugin_property' in params:
            query_params.append(('pluginProperty', params['plugin_property']))  # noqa: E501
            collection_formats['pluginProperty'] = 'multi'  # noqa: E501

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/undoChangePlan', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def update_subscription_bcd(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """Update the BCD associated to a subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_subscription_bcd(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Subscription body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date effective_from_date:
        :param bool force_new_bcd_with_past_effective_date:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.update_subscription_bcd_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
        else:
            (data) = self.update_subscription_bcd_with_http_info(body, x_killbill_created_by, subscription_id, **kwargs)  # noqa: E501
            return data

    def update_subscription_bcd_with_http_info(self, body, x_killbill_created_by, subscription_id, **kwargs):  # noqa: E501
        """Update the BCD associated to a subscription  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_subscription_bcd_with_http_info(body, x_killbill_created_by, subscription_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Subscription body: (required)
        :param str x_killbill_created_by: (required)
        :param str subscription_id: (required)
        :param str x_killbill_reason:
        :param str x_killbill_comment:
        :param date effective_from_date:
        :param bool force_new_bcd_with_past_effective_date:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'x_killbill_created_by', 'subscription_id', 'x_killbill_reason', 'x_killbill_comment', 'effective_from_date', 'force_new_bcd_with_past_effective_date']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_subscription_bcd" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `update_subscription_bcd`")  # noqa: E501
        # verify the required parameter 'x_killbill_created_by' is set
        if ('x_killbill_created_by' not in params or
                params['x_killbill_created_by'] is None):
            raise ValueError("Missing the required parameter `x_killbill_created_by` when calling `update_subscription_bcd`")  # noqa: E501
        # verify the required parameter 'subscription_id' is set
        if ('subscription_id' not in params or
                params['subscription_id'] is None):
            raise ValueError("Missing the required parameter `subscription_id` when calling `update_subscription_bcd`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'subscription_id' in params:
            path_params['subscriptionId'] = params['subscription_id']  # noqa: E501

        query_params = []
        if 'effective_from_date' in params:
            query_params.append(('effectiveFromDate', params['effective_from_date']))  # noqa: E501
        if 'force_new_bcd_with_past_effective_date' in params:
            query_params.append(('forceNewBcdWithPastEffectiveDate', params['force_new_bcd_with_past_effective_date']))  # noqa: E501

        header_params = {}
        if 'x_killbill_created_by' in params:
            header_params['X-Killbill-CreatedBy'] = params['x_killbill_created_by']  # noqa: E501
        if 'x_killbill_reason' in params:
            header_params['X-Killbill-Reason'] = params['x_killbill_reason']  # noqa: E501
        if 'x_killbill_comment' in params:
            header_params['X-Killbill-Comment'] = params['x_killbill_comment']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/subscriptions/{subscriptionId}/bcd', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    Kill Bill

    Kill Bill is an open-source billing and payments platform  # noqa: E501

    OpenAPI spec version: 0.22.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class InvoiceDryRun(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'dry_run_type': 'str',
        'dry_run_action': 'str',
        'phase_type': 'str',
        'product_name': 'str',
        'product_category': 'str',
        'billing_period': 'str',
        'price_list_name': 'str',
        'subscription_id': 'str',
        'bundle_id': 'str',
        'effective_date': 'date',
        'billing_policy': 'str',
        'price_overrides': 'list[PhasePrice]'
    }

    attribute_map = {
        'dry_run_type': 'dryRunType',
        'dry_run_action': 'dryRunAction',
        'phase_type': 'phaseType',
        'product_name': 'productName',
        'product_category': 'productCategory',
        'billing_period': 'billingPeriod',
        'price_list_name': 'priceListName',
        'subscription_id': 'subscriptionId',
        'bundle_id': 'bundleId',
        'effective_date': 'effectiveDate',
        'billing_policy': 'billingPolicy',
        'price_overrides': 'priceOverrides'
    }

    def __init__(self, dry_run_type=None, dry_run_action=None, phase_type=None, product_name=None, product_category=None, billing_period=None, price_list_name=None, subscription_id=None, bundle_id=None, effective_date=None, billing_policy=None, price_overrides=None):  # noqa: E501
        """InvoiceDryRun - a model defined in Swagger"""  # noqa: E501
        self._dry_run_type = None
        self._dry_run_action = None
        self._phase_type = None
        self._product_name = None
        self._product_category = None
        self._billing_period = None
        self._price_list_name = None
        self._subscription_id = None
        self._bundle_id = None
        self._effective_date = None
        self._billing_policy = None
        self._price_overrides = None
        self.discriminator = None
        if dry_run_type is not None:
            self.dry_run_type = dry_run_type
        if dry_run_action is not None:
            self.dry_run_action = dry_run_action
        if phase_type is not None:
            self.phase_type = phase_type
        if product_name is not None:
            self.product_name = product_name
        if product_category is not None:
            self.product_category = product_category
        if billing_period is not None:
            self.billing_period = billing_period
        if price_list_name is not None:
            self.price_list_name = price_list_name
        if subscription_id is not None:
            self.subscription_id = subscription_id
        if bundle_id is not None:
            self.bundle_id = bundle_id
        if effective_date is not None:
            self.effective_date = effective_date
        if billing_policy is not None:
            self.billing_policy = billing_policy
        if price_overrides is not None:
            self.price_overrides = price_overrides

    @property
    def dry_run_type(self):
        """Gets the dry_run_type of this InvoiceDryRun.  # noqa: E501


        :return: The dry_run_type of this InvoiceDryRun.  # noqa: E501
        :rtype: str
        """
        return self._dry_run_type

    @dry_run_type.setter
    def dry_run_type(self, dry_run_type):
        """Sets the dry_run_type of this InvoiceDryRun.


        :param dry_run_type: The dry_run_type of this InvoiceDryRun.  # noqa: E501
        :type: str
        """
        allowed_values = ["TARGET_DATE", "UPCOMING_INVOICE", "SUBSCRIPTION_ACTION"]  # noqa: E501
        if dry_run_type not in allowed_values:
            raise ValueError(
                "Invalid value for `dry_run_type` ({0}), must be one of {1}"  # noqa: E501
                .format(dry_run_type, allowed_values)
            )

        self._dry_run_type = dry_run_type

    @property
    def dry_run_action(self):
        """Gets the dry_run_action of this InvoiceDryRun.  # noqa: E501


        :return: The dry_run_action of this InvoiceDryRun.  # noqa: E501
        :rtype: str
        """
        return self._dry_run_action

    @dry_run_action.setter
    def dry_run_action(self, dry_run_action):
        """Sets the dry_run_action of this InvoiceDryRun.


        :param dry_run_action: The dry_run_action of this InvoiceDryRun.  # noqa: E501
        :type: str
        """
        allowed_values = ["START_ENTITLEMENT", "START_BILLING", "PAUSE_ENTITLEMENT", "PAUSE_BILLING", "RESUME_ENTITLEMENT", "RESUME_BILLING", "PHASE", "CHANGE", "STOP_ENTITLEMENT", "STOP_BILLING", "SERVICE_STATE_CHANGE"]  # noqa: E501
        if dry_run_action not in allowed_values:
            raise ValueError(
                "Invalid value for `dry_run_action` ({0}), must be one of {1}"  # noqa: E501
                .format(dry_run_action, allowed_values)
            )

        self._dry_run_action = dry_run_action

    @property
    def phase_type(self):
        """Gets the phase_type of this InvoiceDryRun.  # noqa: E501


        :return: The phase_type of this InvoiceDryRun.  # noqa: E501
        :rtype: str
        """
        return self._phase_type

    @phase_type.setter
    def phase_type(self, phase_type):
        """Sets the phase_type of this InvoiceDryRun.


        :param phase_type: The phase_type of this InvoiceDryRun.  # noqa: E501
        :type: str
        """
        allowed_values = ["TRIAL", "DISCOUNT", "FIXEDTERM", "EVERGREEN"]  # noqa: E501
        if phase_type not in allowed_values:
            raise ValueError(
                "Invalid value for `phase_type` ({0}), must be one of {1}"  # noqa: E501
                .format(phase_type, allowed_values)
            )

        self._phase_type = phase_type

    @property
    def product_name(self):
        """Gets the product_name of this InvoiceDryRun.  # noqa: E501


        :return: The product_name of this InvoiceDryRun.  # noqa: E501
        :rtype: str
        """
        return self._product_name

    @product_name.setter
    def product_name(self, product_name):
        """Sets the product_name of this InvoiceDryRun.


        :param product_name: The product_name of this InvoiceDryRun.  # noqa: E501
        :type: str
        """

        self._product_name = product_name

    @property
    def product_category(self):
        """Gets the product_category of this InvoiceDryRun.  # noqa: E501


        :return: The product_category of this InvoiceDryRun.  # noqa: E501
        :rtype: str
        """
        return self._product_category

    @product_category.setter
    def product_category(self, product_category):
        """Sets the product_category of this InvoiceDryRun.


        :param product_category: The product_category of this InvoiceDryRun.  # noqa: E501
        :type: str
        """
        allowed_values = ["BASE", "ADD_ON", "STANDALONE"]  # noqa: E501
        if product_category not in allowed_values:
            raise ValueError(
                "Invalid value for `product_category` ({0}), must be one of {1}"  # noqa: E501
                .format(product_category, allowed_values)
            )

        self._product_category = product_category

    @property
    def billing_period(self):
        """Gets the billing_period of this InvoiceDryRun.  # noqa: E501


        :return: The billing_period of this InvoiceDryRun.  # noqa: E501
        :rtype: str
        """
        return self._billing_period

    @billing_period.setter
    def billing_period(self, billing_period):
        """Sets the billing_period of this InvoiceDryRun.


        :param billing_period: The billing_period of this InvoiceDryRun.  # noqa: E501
        :type: str
        """
        allowed_values = ["DAILY", "WEEKLY", "BIWEEKLY", "THIRTY_DAYS", "SIXTY_DAYS", "NINETY_DAYS", "MONTHLY", "BIMESTRIAL", "QUARTERLY", "TRIANNUAL", "BIANNUAL", "ANNUAL", "BIENNIAL", "NO_BILLING_PERIOD"]  # noqa: E501
        if billing_period not in allowed_values:
            raise ValueError(
                "Invalid value for `billing_period` ({0}), must be one of {1}"  # noqa: E501
                .format(billing_period, allowed_values)
            )

        self._billing_period = billing_period

    @property
    def price_list_name(self):
        """Gets the price_list_name of this InvoiceDryRun.  # noqa: E501


        :return: The price_list_name of this InvoiceDryRun.  # noqa: E501
        :rtype: str
        """
        return self._price_list_name

    @price_list_name.setter
    def price_list_name(self, price_list_name):
        """Sets the price_list_name of this InvoiceDryRun.


        :param price_list_name: The price_list_name of this InvoiceDryRun.  # noqa: E501
        :type: str
        """

        self._price_list_name = price_list_name

    @property
    def subscription_id(self):
        """Gets the subscription_id of this InvoiceDryRun.  # noqa: E501


        :return: The subscription_id of this InvoiceDryRun.  # noqa: E501
        :rtype: str
        """
        return self._subscription_id

    @subscription_id.setter
    def subscription_id(self, subscription_id):
        """Sets the subscription_id of this InvoiceDryRun.


        :param subscription_id: The subscription_id of this InvoiceDryRun.  # noqa: E501
        :type: str
        """

        self._subscription_id = subscription_id

    @property
    def bundle_id(self):
        """Gets the bundle_id of this InvoiceDryRun.  # noqa: E501


        :return: The bundle_id of this InvoiceDryRun.  # noqa: E501
        :rtype: str
        """
        return self._bundle_id

    @bundle_id.setter
    def bundle_id(self, bundle_id):
        """Sets the bundle_id of this InvoiceDryRun.


        :param bundle_id: The bundle_id of this InvoiceDryRun.  # noqa: E501
        :type: str
        """

        self._bundle_id = bundle_id

    @property
    def effective_date(self):
        """Gets the effective_date of this InvoiceDryRun.  # noqa: E501


        :return: The effective_date of this InvoiceDryRun.  # noqa: E501
        :rtype: date
        """
        return self._effective_date

    @effective_date.setter
    def effective_date(self, effective_date):
        """Sets the effective_date of this InvoiceDryRun.


        :param effective_date: The effective_date of this InvoiceDryRun.  # noqa: E501
        :type: date
        """

        self._effective_date = effective_date

    @property
    def billing_policy(self):
        """Gets the billing_policy of this InvoiceDryRun.  # noqa: E501


        :return: The billing_policy of this InvoiceDryRun.  # noqa: E501
        :rtype: str
        """
        return self._billing_policy

    @billing_policy.setter
    def billing_policy(self, billing_policy):
        """Sets the billing_policy of this InvoiceDryRun.


        :param billing_policy: The billing_policy of this InvoiceDryRun.  # noqa: E501
        :type: str
        """
        allowed_values = ["START_OF_TERM", "END_OF_TERM", "IMMEDIATE", "ILLEGAL"]  # noqa: E501
        if billing_policy not in allowed_values:
            raise ValueError(
                "Invalid value for `billing_policy` ({0}), must be one of {1}"  # noqa: E501
                .format(billing_policy, allowed_values)
            )

        self._billing_policy = billing_policy

    @property
    def price_overrides(self):
        """Gets the price_overrides of this InvoiceDryRun.  # noqa: E501


        :return: The price_overrides of this InvoiceDryRun.  # noqa: E501
        :rtype: list[PhasePrice]
        """
        return self._price_overrides

    @price_overrides.setter
    def price_overrides(self, price_overrides):
        """Sets the price_overrides of this InvoiceDryRun.


        :param price_overrides: The price_overrides of this InvoiceDryRun.  # noqa: E501
        :type: list[PhasePrice]
        """

        self._price_overrides = price_overrides

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(InvoiceDryRun, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, InvoiceDryRun):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Kill Bill

    Kill Bill is an open-source billing and payments platform  # noqa: E501

    OpenAPI spec version: 0.22.22-SNAPSHOT
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import killbill
from killbill.api.bundle_api import BundleApi  # noqa: E501
from killbill.rest import ApiException


class TestBundleApi(unittest.TestCase):
    """BundleApi unit test stubs"""

    def setUp(self):
        self.api = killbill.api.bundle_api.BundleApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_add_bundle_blocking_state(self):
        """Test case for add_bundle_blocking_state

        Block a bundle  # noqa: E501
        """
        pass

    def test_create_bundle_custom_fields(self):
        """Test case for create_bundle_custom_fields

        Add custom fields to bundle  # noqa: E501
        """
        pass

    def test_create_bundle_tags(self):
        """Test case for create_bundle_tags

        Add tags to bundle  # noqa: E501
        """
        pass

    def test_delete_bundle_custom_fields(self):
        """Test case for delete_bundle_custom_fields

        Remove custom fields from bundle  # noqa: E501
        """
        pass

    def test_delete_bundle_tags(self):
        """Test case for delete_bundle_tags

        Remove tags from bundle  # noqa: E501
        """
        pass

    def test_get_bundle(self):
        """Test case for get_bundle

        Retrieve a bundle by id  # noqa: E501
        """
        pass

    def test_get_bundle_audit_logs_with_history(self):
        """Test case for get_bundle_audit_logs_with_history

        Retrieve bundle audit logs with history by id  # noqa: E501
        """
        pass

    def test_get_bundle_by_key(self):
        """Test case for get_bundle_by_key

        Retrieve a bundle by external key  # noqa: E501
        """
        pass

    def test_get_bundle_custom_fields(self):
        """Test case for get_bundle_custom_fields

        Retrieve bundle custom fields  # noqa: E501
        """
        pass

    def test_get_bundle_tags(self):
        """Test case for get_bundle_tags

        Retrieve bundle tags  # noqa: E501
        """
        pass

    def test_get_bundles(self):
        """Test case for get_bundles

        List bundles  # noqa: E501
        """
        pass

    def test_modify_bundle_custom_fields(self):
        """Test case for modify_bundle_custom_fields

        Modify custom fields to bundle  # noqa: E501
        """
        pass

    def test_pause_bundle(self):
        """Test case for pause_bundle

        Pause a bundle  # noqa: E501
        """
        pass

    def test_rename_external_key(self):
        """Test case for rename_external_key

        Update a bundle externalKey  # noqa: E501
        """
        pass

    def test_resume_bundle(self):
        """Test case for resume_bundle

        Resume a bundle  # noqa: E501
        """
        pass

    def test_search_bundles(self):
        """Test case for search_bundles

        Search bundles  # noqa: E501
        """
        pass

    def test_transfer_bundle(self):
        """Test case for transfer_bundle

        Transfer a bundle to another account  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kill Bill

    Kill Bill is an open-source billing and payments platform  # noqa: E501

    OpenAPI spec version: 0.22.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class AuditLog(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'change_type': 'str',
        'change_date': 'datetime',
        'object_type': 'str',
        'object_id': 'str',
        'changed_by': 'str',
        'reason_code': 'str',
        'comments': 'str',
        'user_token': 'str',
        'history': 'Entity'
    }

    attribute_map = {
        'change_type': 'changeType',
        'change_date': 'changeDate',
        'object_type': 'objectType',
        'object_id': 'objectId',
        'changed_by': 'changedBy',
        'reason_code': 'reasonCode',
        'comments': 'comments',
        'user_token': 'userToken',
        'history': 'history'
    }

    def __init__(self, change_type=None, change_date=None, object_type=None, object_id=None, changed_by=None, reason_code=None, comments=None, user_token=None, history=None):  # noqa: E501
        """AuditLog - a model defined in Swagger"""  # noqa: E501
        self._change_type = None
        self._change_date = None
        self._object_type = None
        self._object_id = None
        self._changed_by = None
        self._reason_code = None
        self._comments = None
        self._user_token = None
        self._history = None
        self.discriminator = None
        if change_type is not None:
            self.change_type = change_type
        if change_date is not None:
            self.change_date = change_date
        if object_type is not None:
            self.object_type = object_type
        if object_id is not None:
            self.object_id = object_id
        if changed_by is not None:
            self.changed_by = changed_by
        if reason_code is not None:
            self.reason_code = reason_code
        if comments is not None:
            self.comments = comments
        if user_token is not None:
            self.user_token = user_token
        if history is not None:
            self.history = history

    @property
    def change_type(self):
        """Gets the change_type of this AuditLog.  # noqa: E501


        :return: The change_type of this AuditLog.  # noqa: E501
        :rtype: str
        """
        return self._change_type

    @change_type.setter
    def change_type(self, change_type):
        """Sets the change_type of this AuditLog.


        :param change_type: The change_type of this AuditLog.  # noqa: E501
        :type: str
        """

        self._change_type = change_type

    @property
    def change_date(self):
        """Gets the change_date of this AuditLog.  # noqa: E501


        :return: The change_date of this AuditLog.  # noqa: E501
        :rtype: datetime
        """
        return self._change_date

    @change_date.setter
    def change_date(self, change_date):
        """Sets the change_date of this AuditLog.


        :param change_date: The change_date of this AuditLog.  # noqa: E501
        :type: datetime
        """

        self._change_date = change_date

    @property
    def object_type(self):
        """Gets the object_type of this AuditLog.  # noqa: E501


        :return: The object_type of this AuditLog.  # noqa: E501
        :rtype: str
        """
        return self._object_type

    @object_type.setter
    def object_type(self, object_type):
        """Sets the object_type of this AuditLog.


        :param object_type: The object_type of this AuditLog.  # noqa: E501
        :type: str
        """
        allowed_values = ["ACCOUNT", "ACCOUNT_EMAIL", "BLOCKING_STATES", "BUNDLE", "CUSTOM_FIELD", "INVOICE", "PAYMENT", "TRANSACTION", "INVOICE_ITEM", "INVOICE_PAYMENT", "SUBSCRIPTION", "SUBSCRIPTION_EVENT", "SERVICE_BROADCAST", "PAYMENT_ATTEMPT", "PAYMENT_METHOD", "TAG", "TAG_DEFINITION", "TENANT", "TENANT_KVS"]  # noqa: E501
        if object_type not in allowed_values:
            raise ValueError(
                "Invalid value for `object_type` ({0}), must be one of {1}"  # noqa: E501
                .format(object_type, allowed_values)
            )

        self._object_type = object_type

    @property
    def object_id(self):
        """Gets the object_id of this AuditLog.  # noqa: E501


        :return: The object_id of this AuditLog.  # noqa: E501
        :rtype: str
        """
        return self._object_id

    @object_id.setter
    def object_id(self, object_id):
        """Sets the object_id of this AuditLog.


        :param object_id: The object_id of this AuditLog.  # noqa: E501
        :type: str
        """

        self._object_id = object_id

    @property
    def changed_by(self):
        """Gets the changed_by of this AuditLog.  # noqa: E501


        :return: The changed_by of this AuditLog.  # noqa: E501
        :rtype: str
        """
        return self._changed_by

    @changed_by.setter
    def changed_by(self, changed_by):
        """Sets the changed_by of this AuditLog.


        :param changed_by: The changed_by of this AuditLog.  # noqa: E501
        :type: str
        """

        self._changed_by = changed_by

    @property
    def reason_code(self):
        """Gets the reason_code of this AuditLog.  # noqa: E501


        :return: The reason_code of this AuditLog.  # noqa: E501
        :rtype: str
        """
        return self._reason_code

    @reason_code.setter
    def reason_code(self, reason_code):
        """Sets the reason_code of this AuditLog.


        :param reason_code: The reason_code of this AuditLog.  # noqa: E501
        :type: str
        """

        self._reason_code = reason_code

    @property
    def comments(self):
        """Gets the comments of this AuditLog.  # noqa: E501


        :return: The comments of this AuditLog.  # noqa: E501
        :rtype: str
        """
        return self._comments

    @comments.setter
    def comments(self, comments):
        """Sets the comments of this AuditLog.


        :param comments: The comments of this AuditLog.  # noqa: E501
        :type: str
        """

        self._comments = comments

    @property
    def user_token(self):
        """Gets the user_token of this AuditLog.  # noqa: E501


        :return: The user_token of this AuditLog.  # noqa: E501
        :rtype: str
        """
        return self._user_token

    @user_token.setter
    def user_token(self, user_token):
        """Sets the user_token of this AuditLog.


        :param user_token: The user_token of this AuditLog.  # noqa: E501
        :type: str
        """

        self._user_token = user_token

    @property
    def history(self):
        """Gets the history of this AuditLog.  # noqa: E501


        :return: The history of this AuditLog.  # noqa: E501
        :rtype: Entity
        """
        return self._history

    @history.setter
    def history(self, history):
        """Sets the history of this AuditLog.


        :param history: The history of this AuditLog.  # noqa: E501
        :type: Entity
        """

        self._history = history

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(AuditLog, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AuditLog):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Kill Bill

    Kill Bill is an open-source billing and payments platform  # noqa: E501

    OpenAPI spec version: 0.22.22-SNAPSHOT
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import killbill
from killbill.api.security_api import SecurityApi  # noqa: E501
from killbill.rest import ApiException


class TestSecurityApi(unittest.TestCase):
    """SecurityApi unit test stubs"""

    def setUp(self):
        self.api = killbill.api.security_api.SecurityApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_add_role_definition(self):
        """Test case for add_role_definition

        Add a new role definition)  # noqa: E501
        """
        pass

    def test_add_user_roles(self):
        """Test case for add_user_roles

        Add a new user with roles (to make api requests)  # noqa: E501
        """
        pass

    def test_get_current_user_permissions(self):
        """Test case for get_current_user_permissions

        List user permissions  # noqa: E501
        """
        pass

    def test_get_current_user_subject(self):
        """Test case for get_current_user_subject

        Get user information  # noqa: E501
        """
        pass

    def test_get_role_definition(self):
        """Test case for get_role_definition

        Get role definition  # noqa: E501
        """
        pass

    def test_get_user_roles(self):
        """Test case for get_user_roles

        Get roles associated to a user  # noqa: E501
        """
        pass

    def test_invalidate_user(self):
        """Test case for invalidate_user

        Invalidate an existing user  # noqa: E501
        """
        pass

    def test_update_role_definition(self):
        """Test case for update_role_definition

        Update a new role definition)  # noqa: E501
        """
        pass

    def test_update_user_password(self):
        """Test case for update_user_password

        Update a user password  # noqa: E501
        """
        pass

    def test_update_user_roles(self):
        """Test case for update_user_roles

        Update roles associated to a user  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kill Bill

    Kill Bill is an open-source billing and payments platform  # noqa: E501

    OpenAPI spec version: 0.22.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from killbill.api_client import ApiClient


class CustomFieldApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def get_custom_field_audit_logs_with_history(self, custom_field_id, **kwargs):  # noqa: E501
        """Retrieve custom field audit logs with history by id  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_custom_field_audit_logs_with_history(custom_field_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str custom_field_id: (required)
        :return: list[AuditLog]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_custom_field_audit_logs_with_history_with_http_info(custom_field_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_custom_field_audit_logs_with_history_with_http_info(custom_field_id, **kwargs)  # noqa: E501
            return data

    def get_custom_field_audit_logs_with_history_with_http_info(self, custom_field_id, **kwargs):  # noqa: E501
        """Retrieve custom field audit logs with history by id  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_custom_field_audit_logs_with_history_with_http_info(custom_field_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str custom_field_id: (required)
        :return: list[AuditLog]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['custom_field_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_custom_field_audit_logs_with_history" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'custom_field_id' is set
        if ('custom_field_id' not in params or
                params['custom_field_id'] is None):
            raise ValueError("Missing the required parameter `custom_field_id` when calling `get_custom_field_audit_logs_with_history`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'custom_field_id' in params:
            path_params['customFieldId'] = params['custom_field_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/customFields/{customFieldId}/auditLogsWithHistory', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[AuditLog]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_custom_fields(self, **kwargs):  # noqa: E501
        """List custom fields  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_custom_fields(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int offset:
        :param int limit:
        :param str audit:
        :return: list[CustomField]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_custom_fields_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.get_custom_fields_with_http_info(**kwargs)  # noqa: E501
            return data

    def get_custom_fields_with_http_info(self, **kwargs):  # noqa: E501
        """List custom fields  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_custom_fields_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int offset:
        :param int limit:
        :param str audit:
        :return: list[CustomField]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['offset', 'limit', 'audit']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_custom_fields" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'audit' in params:
            query_params.append(('audit', params['audit']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/customFields/pagination', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[CustomField]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def search_custom_fields(self, search_key, **kwargs):  # noqa: E501
        """Search custom fields  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.search_custom_fields(search_key, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str search_key: (required)
        :param int offset:
        :param int limit:
        :param str audit:
        :return: list[CustomField]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.search_custom_fields_with_http_info(search_key, **kwargs)  # noqa: E501
        else:
            (data) = self.search_custom_fields_with_http_info(search_key, **kwargs)  # noqa: E501
            return data

    def search_custom_fields_with_http_info(self, search_key, **kwargs):  # noqa: E501
        """Search custom fields  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.search_custom_fields_with_http_info(search_key, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str search_key: (required)
        :param int offset:
        :param int limit:
        :param str audit:
        :return: list[CustomField]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['search_key', 'offset', 'limit', 'audit']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method search_custom_fields" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'search_key' is set
        if ('search_key' not in params or
                params['search_key'] is None):
            raise ValueError("Missing the required parameter `search_key` when calling `search_custom_fields`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'search_key' in params:
            path_params['searchKey'] = params['search_key']  # noqa: E501

        query_params = []
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'audit' in params:
            query_params.append(('audit', params['audit']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/customFields/search/{searchKey}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[CustomField]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def search_custom_fields_by_type_name(self, **kwargs):  # noqa: E501
        """Search custom fields by type, name and optional value  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.search_custom_fields_by_type_name(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str object_type:
        :param str field_name:
        :param str field_value:
        :param int offset:
        :param int limit:
        :param str audit:
        :return: list[CustomField]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.search_custom_fields_by_type_name_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.search_custom_fields_by_type_name_with_http_info(**kwargs)  # noqa: E501
            return data

    def search_custom_fields_by_type_name_with_http_info(self, **kwargs):  # noqa: E501
        """Search custom fields by type, name and optional value  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.search_custom_fields_by_type_name_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str object_type:
        :param str field_name:
        :param str field_value:
        :param int offset:
        :param int limit:
        :param str audit:
        :return: list[CustomField]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['object_type', 'field_name', 'field_value', 'offset', 'limit', 'audit']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method search_custom_fields_by_type_name" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'object_type' in params:
            query_params.append(('objectType', params['object_type']))  # noqa: E501
        if 'field_name' in params:
            query_params.append(('fieldName', params['field_name']))  # noqa: E501
        if 'field_value' in params:
            query_params.append(('fieldValue', params['field_value']))  # noqa: E501
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'audit' in params:
            query_params.append(('audit', params['audit']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Killbill Api Key', 'Killbill Api Secret', 'basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/1.0/kb/customFields/search', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[CustomField]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    Kill Bill

    Kill Bill is an open-source billing and payments platform  # noqa: E501

    OpenAPI spec version: 0.22.22-SNAPSHOT
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import killbill
from killbill.api.tag_api import TagApi  # noqa: E501
from killbill.rest import ApiException


class TestTagApi(unittest.TestCase):
    """TagApi unit test stubs"""

    def setUp(self):
        self.api = killbill.api.tag_api.TagApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_get_tag_audit_logs_with_history(self):
        """Test case for get_tag_audit_logs_with_history

        Retrieve tag audit logs with history by id  # noqa: E501
        """
        pass

    def test_get_tags(self):
        """Test case for get_tags

        List tags  # noqa: E501
        """
        pass

    def test_search_tags(self):
        """Test case for search_tags

        Search tags  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kill Bill

    Kill Bill is an open-source billing and payments platform  # noqa: E501

    OpenAPI spec version: 0.22.22-SNAPSHOT
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import killbill
from killbill.api.tenant_api import TenantApi  # noqa: E501
from killbill.rest import ApiException


class TestTenantApi(unittest.TestCase):
    """TenantApi unit test stubs"""

    def setUp(self):
        self.api = killbill.api.tenant_api.TenantApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_create_tenant(self):
        """Test case for create_tenant

        Create a tenant  # noqa: E501
        """
        pass

    def test_delete_per_tenant_configuration(self):
        """Test case for delete_per_tenant_configuration

        Delete a per tenant configuration (system properties)  # noqa: E501
        """
        pass

    def test_delete_plugin_configuration(self):
        """Test case for delete_plugin_configuration

        Delete a per tenant configuration for a plugin  # noqa: E501
        """
        pass

    def test_delete_plugin_payment_state_machine_config(self):
        """Test case for delete_plugin_payment_state_machine_config

        Delete a per tenant payment state machine for a plugin  # noqa: E501
        """
        pass

    def test_delete_push_notification_callbacks(self):
        """Test case for delete_push_notification_callbacks

        Delete a push notification  # noqa: E501
        """
        pass

    def test_delete_user_key_value(self):
        """Test case for delete_user_key_value

        Delete  a per tenant user key/value  # noqa: E501
        """
        pass

    def test_get_all_plugin_configuration(self):
        """Test case for get_all_plugin_configuration

        Retrieve a per tenant key value based on key prefix  # noqa: E501
        """
        pass

    def test_get_per_tenant_configuration(self):
        """Test case for get_per_tenant_configuration

        Retrieve a per tenant configuration (system properties)  # noqa: E501
        """
        pass

    def test_get_plugin_configuration(self):
        """Test case for get_plugin_configuration

        Retrieve a per tenant configuration for a plugin  # noqa: E501
        """
        pass

    def test_get_plugin_payment_state_machine_config(self):
        """Test case for get_plugin_payment_state_machine_config

        Retrieve a per tenant payment state machine for a plugin  # noqa: E501
        """
        pass

    def test_get_push_notification_callbacks(self):
        """Test case for get_push_notification_callbacks

        Retrieve a push notification  # noqa: E501
        """
        pass

    def test_get_tenant(self):
        """Test case for get_tenant

        Retrieve a tenant by id  # noqa: E501
        """
        pass

    def test_get_tenant_by_api_key(self):
        """Test case for get_tenant_by_api_key

        Retrieve a tenant by its API key  # noqa: E501
        """
        pass

    def test_get_user_key_value(self):
        """Test case for get_user_key_value

        Retrieve a per tenant user key/value  # noqa: E501
        """
        pass

    def test_insert_user_key_value(self):
        """Test case for insert_user_key_value

        Add a per tenant user key/value  # noqa: E501
        """
        pass

    def test_register_push_notification_callback(self):
        """Test case for register_push_notification_callback

        Create a push notification  # noqa: E501
        """
        pass

    def test_upload_per_tenant_configuration(self):
        """Test case for upload_per_tenant_configuration

        Add a per tenant configuration (system properties)  # noqa: E501
        """
        pass

    def test_upload_plugin_configuration(self):
        """Test case for upload_plugin_configuration

        Add a per tenant configuration for a plugin  # noqa: E501
        """
        pass

    def test_upload_plugin_payment_state_machine_config(self):
        """Test case for upload_plugin_payment_state_machine_config

        Add a per tenant payment state machine for a plugin  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kill Bill

    Kill Bill is an open-source billing and payments platform  # noqa: E501

    OpenAPI spec version: 0.22.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PhasePrice(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'plan_name': 'str',
        'phase_name': 'str',
        'phase_type': 'str',
        'fixed_price': 'float',
        'recurring_price': 'float',
        'usage_prices': 'list[UsagePrice]'
    }

    attribute_map = {
        'plan_name': 'planName',
        'phase_name': 'phaseName',
        'phase_type': 'phaseType',
        'fixed_price': 'fixedPrice',
        'recurring_price': 'recurringPrice',
        'usage_prices': 'usagePrices'
    }

    def __init__(self, plan_name=None, phase_name=None, phase_type=None, fixed_price=None, recurring_price=None, usage_prices=None):  # noqa: E501
        """PhasePrice - a model defined in Swagger"""  # noqa: E501
        self._plan_name = None
        self._phase_name = None
        self._phase_type = None
        self._fixed_price = None
        self._recurring_price = None
        self._usage_prices = None
        self.discriminator = None
        if plan_name is not None:
            self.plan_name = plan_name
        if phase_name is not None:
            self.phase_name = phase_name
        if phase_type is not None:
            self.phase_type = phase_type
        if fixed_price is not None:
            self.fixed_price = fixed_price
        if recurring_price is not None:
            self.recurring_price = recurring_price
        if usage_prices is not None:
            self.usage_prices = usage_prices

    @property
    def plan_name(self):
        """Gets the plan_name of this PhasePrice.  # noqa: E501


        :return: The plan_name of this PhasePrice.  # noqa: E501
        :rtype: str
        """
        return self._plan_name

    @plan_name.setter
    def plan_name(self, plan_name):
        """Sets the plan_name of this PhasePrice.


        :param plan_name: The plan_name of this PhasePrice.  # noqa: E501
        :type: str
        """

        self._plan_name = plan_name

    @property
    def phase_name(self):
        """Gets the phase_name of this PhasePrice.  # noqa: E501


        :return: The phase_name of this PhasePrice.  # noqa: E501
        :rtype: str
        """
        return self._phase_name

    @phase_name.setter
    def phase_name(self, phase_name):
        """Sets the phase_name of this PhasePrice.


        :param phase_name: The phase_name of this PhasePrice.  # noqa: E501
        :type: str
        """

        self._phase_name = phase_name

    @property
    def phase_type(self):
        """Gets the phase_type of this PhasePrice.  # noqa: E501


        :return: The phase_type of this PhasePrice.  # noqa: E501
        :rtype: str
        """
        return self._phase_type

    @phase_type.setter
    def phase_type(self, phase_type):
        """Sets the phase_type of this PhasePrice.


        :param phase_type: The phase_type of this PhasePrice.  # noqa: E501
        :type: str
        """

        self._phase_type = phase_type

    @property
    def fixed_price(self):
        """Gets the fixed_price of this PhasePrice.  # noqa: E501


        :return: The fixed_price of this PhasePrice.  # noqa: E501
        :rtype: float
        """
        return self._fixed_price

    @fixed_price.setter
    def fixed_price(self, fixed_price):
        """Sets the fixed_price of this PhasePrice.


        :param fixed_price: The fixed_price of this PhasePrice.  # noqa: E501
        :type: float
        """

        self._fixed_price = fixed_price

    @property
    def recurring_price(self):
        """Gets the recurring_price of this PhasePrice.  # noqa: E501


        :return: The recurring_price of this PhasePrice.  # noqa: E501
        :rtype: float
        """
        return self._recurring_price

    @recurring_price.setter
    def recurring_price(self, recurring_price):
        """Sets the recurring_price of this PhasePrice.


        :param recurring_price: The recurring_price of this PhasePrice.  # noqa: E501
        :type: float
        """

        self._recurring_price = recurring_price

    @property
    def usage_prices(self):
        """Gets the usage_prices of this PhasePrice.  # noqa: E501


        :return: The usage_prices of this PhasePrice.  # noqa: E501
        :rtype: list[UsagePrice]
        """
        return self._usage_prices

    @usage_prices.setter
    def usage_prices(self, usage_prices):
        """Sets the usage_prices of this PhasePrice.


        :param usage_prices: The usage_prices of this PhasePrice.  # noqa: E501
        :type: list[UsagePrice]
        """

        self._usage_prices = usage_prices

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PhasePrice, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PhasePrice):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
